<?php

namespace Drupal\cloud\Entity;

use Drupal\cloud\Traits\CloudContentEntityTrait;
use Drupal\Core\Entity\Sql\SqlContentEntityStorage;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Session\AccountInterface;

/**
 * Defines the storage handler class for cloud store entities.
 *
 * This extends the base storage class, adding required special handling for
 * Cloud store entities.
 *
 * @ingroup cloud_store
 */
class CloudStoreStorage extends SqlContentEntityStorage implements CloudStoreStorageInterface {

  use CloudContentEntityTrait;

  /**
   * {@inheritdoc}
   */
  public function revisionIds(CloudStoreInterface $entity): array {
    try {
      return $this->database->query(
        'SELECT vid FROM {cloud_store_revision} WHERE id=:id ORDER BY vid',
        [':id' => $entity->id()]
      )->fetchCol();
    }
    catch (\Exception $e) {
      $this->handleException($e);
    }
    // Exception return type handling.
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function userRevisionIds(AccountInterface $account): array {
    try {
      return $this->database->query(
        'SELECT vid FROM {cloud_store_field_revision} WHERE uid = :uid ORDER BY vid',
        [':uid' => $account->id()]
      )->fetchCol();
    }
    catch (\Exception $e) {
      $this->handleException($e);
    }
    // Exception return type handling.
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function countDefaultLanguageRevisions(CloudStoreInterface $entity): int {
    try {
      return $this->database->query('SELECT COUNT(*) FROM {cloud_store_field_revision} WHERE id = :id AND default_langcode = 1', [':id' => $entity->id()])
        ->fetchField();
    }
    catch (\Exception $e) {
      $this->handleException($e);
    }
    // Exception return type handling.
    return 0;
  }

  /**
   * {@inheritdoc}
   */
  public function clearRevisionsLanguage(LanguageInterface $language) {
    return $this->database->update('cloud_store_revision')
      ->fields(['langcode' => LanguageInterface::LANGCODE_NOT_SPECIFIED])
      ->condition('langcode', $language->getId())
      ->execute();
  }

}
